//+------------------------------------------------------------------+
//|                                                   Mandelbrot.mq5 |
//|                        Copyright 2010, MetaQuotes Software Corp. |
//|                                              http://www.mql5.com |
//+------------------------------------------------------------------+
#property copyright "Copyright 2010, MetaQuotes Software Corp."
#property link      "http://www.mql5.com"
#property version   "1.00"
#property script_show_inputs

input uint ExtWidth     =512;
input uint ExtHeight    =512;
input uint ExtIterations=20000;

//+------------------------------------------------------------------+
//|  BMP                                               |
//+------------------------------------------------------------------+
struct BitmapHeader
  {
   ushort            type;
   uint              size;
   uint              reserv;
   uint              offbits;

   uint              imgSSize;
   uint              imgWidth;
   uint              imgHeight;
   ushort            imgPlanes;
   ushort            imgBitCount;
   uint              imgCompression;
   uint              imgSizeImage;
   uint              imgXPelsPerMeter;
   uint              imgYPelsPerMeter;
   uint              imgClrUsed;
   uint              imgClrImportant;
  };

//+------------------------------------------------------------------+
//|                                                |
//+------------------------------------------------------------------+
bool SaveBitmapToFile(const string filename,uint &bitmap[],uint width,uint height)
  {
//---  
   int file=FileOpen(filename,FILE_WRITE|FILE_BIN);
   if(file==INVALID_HANDLE)
     {
      Print(__FUNCTION__," error opening '",filename,"'");
      return(false);
     }
//---  
   BitmapHeader info;

   ZeroMemory(info);
   info.type          =0x4d42;
   info.size          =sizeof(info)+width*height*4;
   info.offbits       =sizeof(info);

   info.imgSSize      =40;
   info.imgWidth      =width;
   info.imgHeight     =height;
   info.imgPlanes     =1;
   info.imgBitCount   =32;
   info.imgCompression=0;                // BI_RGB
   info.imgSizeImage  =width*height*4;
//---     
   if(FileWriteStruct(file,info)==sizeof(info))
     {
      if(FileWriteArray(file,bitmap)==ArraySize(bitmap))
        {
         FileClose(file);
         return(true);
        }
     }
//---  ,     
   FileClose(file);
   FileDelete(filename);
   Print(__FUNCTION__," error writting '",filename,"'");
//---  
   return(false);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
uint calcMQL(uint &bitmap[],uint w,uint h,uint max)
  {
   uint   startTime=GetTickCount();
   double x0 = -2;
   double y0 = -0.5;
   double x1 = -1;
   double y1 =  0.5;

   uint   scanline=0;
//---
   for(uint gy=0;gy<h;gy++)
     {
      for(uint gx=0;gx<w;gx++)
        {
         double dx = x0 + gx * (x1-x0) / w;
         double dy = y0 + gy * (y1-y0) / h;

         double x  = 0;
         double y  = 0;
         double xx = 0;
         double yy = 0;
         double xy = 0;
         uint   i=0;
         
         while((xx+yy)<4 && i<max)
           {
            xx = x*x;
            yy = y*y;
            xy = x*y;
            y  = xy+xy+dy;
            x  = xx-yy+dx;
            i++;
           }

         if(i==max)
           {
            //Line[gy].Pixel[gx]=0;
            bitmap[scanline]=0;
           }
         else
           {
            //Line[gy].Pixel[gx]=(int)(((float)i/max)*0xFFFFFF);
            bitmap[scanline]=(int)(((double)i/max)*0xFFFFFF);
           }
         scanline++;
        }
     }
//---
   return(GetTickCount()-startTime);
  }
//+------------------------------------------------------------------+
//| Script program start function                                    |
//+------------------------------------------------------------------+
void OnStart()
  {
   int bitmap[];
//---  
   ArrayResize(bitmap,ExtWidth*ExtWidth,0);
//---  
   uint res=calcMQL(bitmap,ExtWidth,ExtHeight,ExtIterations);
   Print("Time: ",res," ms");
//---    BMP 
   SaveBitmapToFile("Mandelbrot.bmp",bitmap,ExtWidth,ExtHeight);
//---
  }
//+------------------------------------------------------------------+
